import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;

import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JToolBar;

import ru.amse.fedorov.plainsvg.PointLocationListener;
import ru.amse.fedorov.plainsvg.gui.ComponentSelectedList;
import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.gui.SelectionChangedListener;
import ru.amse.fedorov.plainsvg.model.elements.SVGShape;
import ru.amse.fedorov.plainsvg.model.elements.SVGStroked;
import ru.amse.fedorov.plainsvg.presentation.elements.ElementPresentation;

/* package */class PropertiesToolbar extends JToolBar implements
		SelectionChangedListener, PointLocationListener {

	/*
	 * serialVersionUID
	 */
	private static final long serialVersionUID = 1L;

	public static final String COLOR_OF_STROKE = "Color of stroke";

	public static final String COLOR_OF_FILL = "Color of fill";

	private static final int TEN = 10;

	/*
	 * Button answering for the stroke color
	 */
	private final JButton strokeButton = new JButton();

	/*
	 * Button answering for the fill color
	 */
	private final JButton fillButton = new JButton();

	/*
	 * The list of selected elements
	 */
	private final ComponentSelectedList selectedList;

	private final JLabel strokeLabel;

	private final JLabel strokeWidthLabel = new JLabel("Stroke width");

	private final JLabel fillLabel;

	private final JTextField strokeField = new JTextField();

	private final GraphicsComponent comp;

	private final JLabel mouseX = new JLabel();

	private final JLabel mouseY = new JLabel();

	private final JButton okButton = new JButton("OK");

	/**
	 * Creates toolbar with givel list of element.
	 * 
	 * @param list
	 *            list of element
	 * 
	 * @see ru.amse.fedorov.plainsvg.gui.GraphicsComponent
	 */
	/* package */PropertiesToolbar(GraphicsComponent component) {
		comp = component;
		comp.getMousePoint().addMouseLocationListener(this);
		setLayout(new GridLayout(2, 0));
		selectedList = comp.getSelectedList();
		Dimension dimension = new Dimension(60, 25);
		fillButton.setSize(dimension);
		fillButton.setPreferredSize(dimension);
		strokeButton.setSize(dimension);
		strokeButton.setPreferredSize(dimension);
		strokeField.setSize(30, 25);
		strokeField.setPreferredSize(new Dimension(30, 25));

		strokeLabel = new JLabel("Stroke color");
		strokeLabel.setEnabled(false);
		strokeWidthLabel.setEnabled(false);
		strokeField.setEnabled(false);
		fillLabel = new JLabel("Fill color");
		fillLabel.setEnabled(false);
		okButton.setEnabled(false);

		add(new JPan(strokeWidthLabel));
		add(new JPan(strokeLabel));
		add(new JPan(fillLabel));
		add(new JLabel());
		add(new JLabel());
		add(new JLabel());
		add(mouseX);
		add(new JPan(strokeField, okButton));
		add(new JPan(strokeButton));
		add(new JPan(fillButton));
		add(new JLabel());
		add(new JLabel());
		add(new JLabel());
		add(mouseY);

		strokeButton.setEnabled(false);
		fillButton.setEnabled(false);

		strokeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Color color = selectedList.size() == 1 ? ((SVGStroked) selectedList
						.get(0).getModelElement()).getStroke()
						: null;
				Color selectedColor = JColorChooser.showDialog(null,
						COLOR_OF_STROKE, color);
				if (selectedColor == null) {
					selectedColor = ((SVGStroked) selectedList.get(0)
							.getModelElement()).getStroke();
				}
				strokeButton.setBackground(selectedColor);
				for (ElementPresentation<?> elem : selectedList) {
					if (elem.hasStroke()) {
						SVGStroked stroked = (SVGStroked) elem
								.getModelElement();
						stroked.setStroke(selectedColor);
					}
				}
			}
		});

		fillButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Color color = selectedList.size() == 1 ? ((SVGShape) selectedList
						.get(0).getModelElement()).getFill()
						: null;
				Color selectedColor = JColorChooser.showDialog(null,
						COLOR_OF_FILL, color);

				if (selectedColor == null) {
					selectedColor = ((SVGShape) selectedList.get(0)
							.getModelElement()).getFill();
				}
				fillButton.setBackground(selectedColor);

				for (ElementPresentation<?> elem : selectedList) {
					if (elem.hasFill()) {
						SVGShape shape = (SVGShape) elem.getModelElement();
						shape.setFill(selectedColor);
					}
				}
			}
		});
		strokeButton.setToolTipText(COLOR_OF_STROKE);
		fillButton.setToolTipText(COLOR_OF_FILL);
		

		okButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				String text = strokeField.getText();
				try {
					double sw = Double.parseDouble(text);
					for (ElementPresentation<?> elem : selectedList) {
						if (elem.hasStroke()) {
							SVGStroked stroked = (SVGStroked) elem
									.getModelElement();
							stroked.setStrokeWidth(sw);
						}
					}
				} catch (NumberFormatException ex) {
					JOptionPane.showMessageDialog(null, "Invalid number: "
							+ text);
				}
			}

		});
	}

	/**
	 * Fill buttons' background with correspondind colors (fill color and stroke
	 * color).
	 */
	public void selectionChanged() {
		strokeWidthLabel.setEnabled(false);
		strokeLabel.setEnabled(false);
		fillLabel.setEnabled(false);
		strokeButton.setEnabled(false);
		fillButton.setEnabled(false);
		okButton.setEnabled(false);
		strokeField.setText("");
		strokeField.setEnabled(false);

		strokeButton.setBackground(null);
		fillButton.setBackground(null);

		if (selectedList.size() == 1) {
			ElementPresentation<?> elem = selectedList.get(0);
			if (elem.hasStroke()) {
				SVGStroked stroked = (SVGStroked) elem.getModelElement();
				strokeButton.setBackground(stroked.getStroke());
				strokeButton.setEnabled(true);
				strokeLabel.setEnabled(true);
				strokeWidthLabel.setEnabled(true);
				okButton.setEnabled(true);
				strokeField.setEnabled(true);
				strokeField.setText("" + stroked.getStrokeWidth());
			}
			if (elem.hasFill()) {
				SVGShape shape = (SVGShape) elem.getModelElement();
				fillButton.setBackground(shape.getFill());
				fillButton.setEnabled(true);
				fillLabel.setEnabled(true);
			}
		} else if (selectedList.size() > 1) {
			strokeButton.setEnabled(true);
			fillButton.setEnabled(true);
			strokeLabel.setEnabled(true);
			fillLabel.setEnabled(true);
			strokeWidthLabel.setEnabled(true);
			okButton.setEnabled(true);
			strokeField.setEnabled(true);
		}
	}

	public void locationChanged() {
		Point2D p = comp.getMousePoint();
		double x = Math.round(p.getX() * TEN);
		double y = Math.round(p.getY() * TEN);
		x = x / TEN;
		y = y / TEN;
		mouseX.setText("X: " + Double.toString(x));
		mouseY.setText("Y: " + Double.toString(y));
	}

	private class JPan extends JPanel {

		private static final long serialVersionUID = 1L;

		private JPan(JComponent comp) {
			super();
			add(comp);
		}

		private JPan(JComponent comp1, JComponent comp2) {
			super();
			add(comp1, BorderLayout.WEST);
			add(comp2, BorderLayout.EAST);
		}

	}
}
