import java.awt.GridLayout;

import javax.swing.ButtonGroup;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.gui.actions.ChangeToolAction;
import ru.amse.fedorov.plainsvg.tools.ToolStatusListener;

/**
 * The toolbar of the editor.
 */
/* package */class SVGToolBar extends JToolBar {

	private static final long serialVersionUID = 1L;

	/*
	 * Group of tool-buttons.
	 */
	private final ButtonGroup group = new ButtonGroup();

	/**
	 * Initialises all main toolbar's elements.
	 * 
	 * @param component
	 *            Main graphics component
	 * 
	 * @see ru.amse.fedorov.plainsvg.gui.GraphicsComponent
	 */
	/* package */SVGToolBar(GraphicsComponent component) {
		JToggleButton select = new ToolToggleButton(component.getSelectAction());

		JToggleButton addRect = new ToolToggleButton(component
				.getAddRectAction());
		JToggleButton addEllipse = new ToolToggleButton(component
				.getAddEllipseAction());
		JToggleButton addLine = new ToolToggleButton(component
				.getAddLineAction());
		JToggleButton addPolygon = new ToolToggleButton(component
				.getAddPolygonAction());

		setLayout(new GridLayout(0, 1));

		addToolButton(select);
		addToolButton(addRect);
		addToolButton(addEllipse);
		addToolButton(addLine);
		addToolButton(addPolygon);
	}

	/**
	 * Adds given JToggleButton to the main group of buttons and to the
	 * component.
	 * 
	 * @param button
	 *            added JToggleButton
	 */
	public void addToolButton(JToggleButton button) {
		group.add(button);
		add(button);
		button.setText("");
	}

	private class ToolToggleButton extends JToggleButton implements
			ToolStatusListener {

		/**
		 * serialVersionUID
		 */
		private static final long serialVersionUID = 1L;

		/**
		 * Creates tool toggle button.
		 * 
		 * @param a
		 *            base action
		 */
		public ToolToggleButton(ChangeToolAction a) {
			super(a);
			a.getTool().addToolStatusListener(this);
		}

		/**
		 * Action to do if tool's status has been changed.
		 */
		public void doOnStatusChanged() {
			setSelected(!isSelected());
		}

	}

}