package ru.amse.fedorov.plainsvg.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

public class PropertiesDialog extends JDialog implements ComponentListener {

	private static final long serialVersionUID = 1L;

	protected static final double MAX_SIZE = 100000;

	private final GraphicsComponent component;

	private final JTextField widthField = new JTextField();

	private final JTextField heightField = new JTextField();

	private boolean Ok = false;

	public PropertiesDialog(JFrame owner, GraphicsComponent comp) {
		super(owner, "Properties", true);
		component = comp;
		component.addComponentListener(this);
		setSize(150, 160);
		setLocation(200, 200);
		Dimension fieldDimension = new Dimension(50, 30);

		widthField.setSize(fieldDimension);
		widthField.setPreferredSize(fieldDimension);

		heightField.setSize(fieldDimension);
		heightField.setPreferredSize(fieldDimension);
		widthField.setText("" + component.getRealWidth());
		heightField.setText("" + component.getRealHeight());

		JPanel widthPanel = new JPanel();
		widthPanel.add(widthField);
		JPanel heightPanel = new JPanel();
		heightPanel.add(heightField);

		JPanel sizePanel = new JPanel();
		sizePanel.setLayout(new GridLayout(0, 2));
		sizePanel.add(new JLabel("width: "));
		sizePanel.add(widthPanel);
		sizePanel.add(new JLabel("height: "));
		sizePanel.add(heightPanel);
		sizePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory
				.createEtchedBorder(), "Size"));
		add(sizePanel, BorderLayout.CENTER);
		JButton okButton = new JButton("OK");
		okButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				Ok = false;
				String widthString = widthField.getText();
				String heightString = heightField.getText();
				try {
					double w = Double.parseDouble(widthString);
					double h = Double.parseDouble(heightString);
					if ((w <= 0) || (h <= 0)) {
						JOptionPane.showMessageDialog(null,
								"Negative size! Try again");
						return;
					}
					if ((w > MAX_SIZE) || (h > MAX_SIZE)) {
						JOptionPane.showMessageDialog(null,
								"Size is too big! Try again");
						return;
					}
					component.setRealSize(w, h);
					setVisible(false);
					Ok = true;
				} catch (NumberFormatException ex) {
					JOptionPane.showMessageDialog(null,
							"Incorrect size! Try again");
				}

			}

		});
		JPanel southPanel = new JPanel();
		southPanel.add(okButton, BorderLayout.CENTER);
		add(southPanel, BorderLayout.SOUTH);
	}

	public void componentHidden(ComponentEvent e) {
		// Nothing!
	}

	public void componentMoved(ComponentEvent e) {
		// Nothing!
	}

	public void componentResized(ComponentEvent e) {
		widthField.setText("" + component.getRealWidth());
		heightField.setText("" + component.getRealHeight());
	}

	public void componentShown(ComponentEvent e) {
		// Nothing!
	}

	public boolean isOK() {
		return Ok;
	}

}
