package ru.amse.fedorov.plainsvg.gui;

import java.util.ArrayList;
import java.util.List;

import ru.amse.fedorov.plainsvg.NotifyingArrayList;

/**
 * List-based model of selected elements.
 * 
 * @author Alexey Fedorov
 * 
 * @param <E>
 *            type of elements
 */
public class SelectedList<E> extends NotifyingArrayList<E> {

	/*
	 * List of listeners
	 */
	private List<SelectionChangedListener> listenerList = new ArrayList<SelectionChangedListener>();

	private static final long serialVersionUID = 1L;

	/*
	 * This method notifies all subscribed listeners that data model has been
	 * changed
	 */
	protected void firePropertyChanged() {
		for (SelectionChangedListener listener : listenerList) {
			listener.selectionChanged();
		}
	}

	/**
	 * Adds a listener to the list that's notified each time a change to the
	 * data model occurs.
	 * 
	 * @param listener
	 *            the <code>SelectionChangedListener</code> to be added
	 */
	public void addSelectionChangedListener(SelectionChangedListener listener) {
		listenerList.add(listener);
	}

	/**
	 * Removes a listener from the list that's notified each time a change to
	 * the data model occurs.
	 * 
	 * @param listener
	 *            the <code>SelectionChangedListener</code> to be removed
	 */
	public void removeSelectionChangedListener(SelectionChangedListener listener) {
		listenerList.remove(listener);
	}
}
