package ru.amse.fedorov.plainsvg.gui.actions;

import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.InputEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.KeyStroke;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;

/**
 * Changes zoom.
 */
public class ZoomAction extends AbstractAction implements ComponentListener {

	/**
	 * serialVersionUID
	 */
	private static final long serialVersionUID = 1L;

	/*
	 * Current zoom
	 */
	private double zoom;

	private GraphicsComponent component;

	/**
	 * Creates zoom action with given parameters.
	 * 
	 * @param zoom
	 *            new zoom
	 * @param name
	 *            name of action (button title)
	 * @param iconPath
	 *            path to the image contained icon for the action
	 * @param c
	 *            accelerator key for the action
	 */
	public ZoomAction(GraphicsComponent component, double zoom, String name,
			String iconPath, char c) {
		super(name, new ImageIcon(ClassLoader.getSystemResource(iconPath+ ".gif")));
		this.component = component;
		this.component.addComponentListener(this);
		int auxiliary = 100;
		this.zoom = zoom;
		putValue(Action.SHORT_DESCRIPTION, "Zoom coefficient = "
				+ (double) (Math.round(auxiliary * zoom)) / auxiliary);
		putValue(Action.MNEMONIC_KEY, new Integer(c));
		putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(new Integer(c),
				InputEvent.CTRL_MASK));
	}

	/**
	 * Creates zoom action with given parameters.
	 * 
	 * @param zoom
	 *            new zoom
	 * @param name
	 *            name of action (button title)
	 * @param iconPath
	 *            path to the image contained icon for the action
	 * @param c
	 *            accelerator key for the action
	 * @param number
	 *            the displayed mnemonic index of the action
	 */
	public ZoomAction(GraphicsComponent component, double zoom, String name,
			String iconPath, char c, int number) {
		this(component, zoom, name, iconPath, c);
		putValue(Action.DISPLAYED_MNEMONIC_INDEX_KEY, number);
	}

	public void actionPerformed(ActionEvent e) {
		component.scale(zoom);
	}

	public void componentHidden(ComponentEvent e) {
		setEnabled(false);
	}

	public void componentMoved(ComponentEvent e) {
		// Nothing!
	}

	public void componentResized(ComponentEvent e) {
		// Nothing!
	}

	public void componentShown(ComponentEvent e) {
		setEnabled(true);
	}

}
