package ru.amse.fedorov.plainsvg.model.elements;

import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.util.ArrayList;

import ru.amse.fedorov.plainsvg.model.NotifyingAffineTransform;
import ru.amse.fedorov.plainsvg.model.listeners.ElementChangedListener;

/**
 * Base class for all SVG-elements. Contains base methods for all SVG-objects.
 * 
 * @author Alexey Fedorov
 */
public abstract class SVGElement {

	/*
	 * The list of listeners.
	 */
	private ArrayList<ElementChangedListener> listenerList = new ArrayList<ElementChangedListener>(
			1);

	/*
	 * The affine transformation of the element.
	 */
	private NotifyingAffineTransform transform;

	/**
	 * Initialzes affine transformation's matrix as unitary.
	 */
	public SVGElement() {
		transform = new NotifyingAffineTransform() {

			private static final long serialVersionUID = 1L;

			@Override
			protected void fireContentChanged() {
				fireElementChanged();
			}

		};
	}
	
	@Override
	public String toString() {
		return transform.toString();
	}

	/**
	 * @return notifying affine transforation
	 */
	public NotifyingAffineTransform getTransform() {
		return transform;
	}

	/**
	 * @param at
	 *            affine transformation
	 */
	public void setAffineTransform(AffineTransform at) {
		transform.setTransform(at);
		fireElementChanged();
	}

	

	/**
	 * Adds a listener to the list that's notified each time a change to the
	 * element occurs.
	 * 
	 * @param listener
	 *            the <code>ElementChangedListener</code> to be added
	 */
	public void addElementChangedListener(ElementChangedListener listener) {
		listenerList.add(listener);
	}

	/**
	 * Removes a listener from the list that's notified each time a change to
	 * the element occurs.
	 * 
	 * @param l
	 *            the <code>ElementChangedListener</code> to be removed
	 */
	public void removeElementChangedListener(ElementChangedListener l) {
		listenerList.remove(l);
	}

	/**
	 * This method notifies all subscribed listeners that element's state has
	 * been changed.
	 */
	protected void fireElementChanged() {
		for (ElementChangedListener elem : listenerList) {
			elem.elementChanged();
		}
	}
	
	public abstract Point2D getCenter();

}
