package ru.amse.fedorov.plainsvg.model.elements;

import java.awt.Color;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;

/**
 * Presents base line's fields and methods constrained with colors, coordinates
 * and sizes.
 * 
 * @author Alexey Fedorov
 */
public class SVGLine extends SVGStroked {

	private final Line2D line = new Line2D.Double();

	/**
	 * Creates line with given parameters.
	 * 
	 * @param x1
	 *            horizontal coordinate of the first line's end
	 * @param y1
	 *            vertical coordinate of the first line's end
	 * @param x2
	 *            horizontal coordinate of the second line's end
	 * @param y2
	 *            vertical coordinate of the second line's end
	 * @param stroke
	 * @param strokeWidth
	 *            thickness of the line
	 */
	public SVGLine(double x1, double y1, double x2, double y2, Color stroke,
			double strokeWidth) {
		super(stroke, strokeWidth);
		initLine(x1, y1, x2, y2);
	}

	public SVGLine(double x1, double y1, double x2, double y2) {
		super();
		initLine(x1, y1, x2, y2);
	}

	public SVGLine(Point2D point) {
		this(point, point);
	}

	public SVGLine(Point2D p1, Point2D p2) {
		line.setLine(p1, p2);
	}

	private void initLine(double x1, double y1, double x2, double y2) {
		line.setLine(x1, y1, x2, y2);
	}

	public void setLine(Point2D p1, Point2D p2) {
		line.setLine(p1, p2);
		fireElementChanged();
	}

	/**
	 * @return the first line's end
	 */
	public Point2D getPoint1() {
		return line.getP1();
	}

	/**
	 * @return the second line's end
	 */
	public Point2D getPoint2() {
		return line.getP2();
	}

	@Override
	public String toString() {
		return "line[first point=" + getPoint1() + ", second point="
				+ getPoint2() + ", stroke color=" + getStroke()
				+ ", stroke width=" + getStrokeWidth() + "] "
				+ super.toString();
	}

	/**
	 * @return the line
	 */
	public Line2D getLine() {
		return new Line2D.Double(line.getP1(), line.getP2());
	}

	@Override
	public Point2D getCenter() {
		double x = (line.getX1() + line.getX2()) / 2;
		double y = (line.getY1() + line.getY2()) / 2;
		return new Point2D.Double(x, y);
	}

}
