package ru.amse.fedorov.plainsvg.model.elements;

import java.awt.Color;
import java.awt.geom.Point2D;

/**
 * Represented bounded geometrical shapes and contains several methods for
 * working with it.
 * 
 * @author Alexey Fedorov
 */
public abstract class SVGRectangularShape extends SVGShape {

	private double x1;

	private double y1;

	private double x2;

	private double y2;

	/**
	 * 
	 * @param fill
	 *            The colour to use for the interior of the shape
	 * @param stroke
	 *            The colour for the line actually drawn for the shape
	 * @param strokeWidth
	 *            The thickness of the line used to draw the shape
	 */
	public SVGRectangularShape(Color fill, Color stroke, double strokeWidth) {
		super(fill, stroke, strokeWidth);
	}

	public SVGRectangularShape(Color fill) {
		super(fill);
	}

	public SVGRectangularShape() {
		this(null);
	}

	/**
	 * @return the x1
	 */
	public double getX1() {
		return x1;
	}

	/**
	 * @param x1
	 *            the x1 to set
	 */
	public void setX1(double x1) {
		this.x1 = x1;
	}

	/**
	 * @return the x2
	 */
	public double getX2() {
		return x2;
	}

	/**
	 * @param x2
	 *            the x2 to set
	 */
	public void setX2(double x2) {
		this.x2 = x2;
	}

	/**
	 * @return the y1
	 */
	public double getY1() {
		return y1;
	}

	/**
	 * @param y1
	 *            the y1 to set
	 */
	public void setY1(double y1) {
		this.y1 = y1;
	}

	/**
	 * @return the y2
	 */
	public double getY2() {
		return y2;
	}

	/**
	 * @param y2
	 *            the y2 to set
	 */
	public void setY2(double y2) {
		this.y2 = y2;
	}

	public Point2D getCenter() {
		return new Point2D.Double((getX1() + getX2()) / 2,
				(getY1() + getY2()) / 2);
	}

	public void setCoordinates(double x1, double y1, double x2, double y2) {
		this.x1 = x1;
		this.y1 = y1;
		this.x2 = x2;
		this.y2 = y2;
		fireElementChanged();
	}

}
