package ru.amse.fedorov.plainsvg.model.elements;

import java.awt.Color;

/**
 * Base class for all stroked SVG-elements. Contains base methods for stroked
 * SVG-objects.
 * 
 * @author Alexey Fedorov
 */
public abstract class SVGStroked extends SVGElement {

	/*
	 * The colour for the line actually drawn for the rectangle.
	 */
	private Color stroke;

	/*
	 * The thickness of the line used to draw the rectangle.
	 */
	private double strokeWidth;

	/**
	 * Creates strocked svg-element with given color and thickness of border
	 * 
	 * @param stroke
	 *            The colour for the line actually drawn for the rectangle
	 * @param strokeWidth
	 *            The thickness of the line used to draw the rectangle
	 */
	public SVGStroked(Color stroke, double strokeWidth) {
		super();
		this.stroke = stroke;
		this.strokeWidth = Math.abs(strokeWidth);
	}

	/**
	 * Creates strocked svg-element with empty (null) color and single thickness
	 * of border.
	 */
	public SVGStroked() {
		this(Color.BLACK, 1);
	}

	/**
	 * @return color of border
	 */
	public Color getStroke() {
		return stroke;
	}

	/**
	 * Sets the color of this stroked svg-element
	 * 
	 * @param stroke
	 *            color of border
	 */
	public void setStroke(Color stroke) {
		this.stroke = stroke;
		fireElementChanged();
	}

	/**
	 * @return border's thickness
	 */
	public double getStrokeWidth() {
		return strokeWidth;
	}

	/**
	 * @param strokeWidth
	 *            border's thickness
	 */
	public void setStrokeWidth(double strokeWidth) {
		this.strokeWidth = strokeWidth;
		fireElementChanged();
	}

}
