package ru.amse.fedorov.plainsvg.presentation.elements;

import java.awt.Cursor;
import java.awt.Image;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.Toolkit;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.List;

import org.w3c.dom.Element;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.model.elements.SVGPolygon;
import ru.amse.fedorov.plainsvg.presentation.markers.poly.PolyRotateMarker;
import ru.amse.fedorov.plainsvg.presentation.markers.poly.PolygonMarker;

public class PolygonPresentation extends ShapePresentation<SVGPolygon> {

	private final SVGPolygon svgPolygon;

	private final Polygon javaPolygon = new Polygon();

	private final Rectangle2D minRect = new Rectangle2D.Double();

	public PolygonPresentation(SVGPolygon polygon, GraphicsComponent comp) {
		super(comp);
		svgPolygon = polygon;
		for (int i = 0; i < svgPolygon.getPointList().size(); i++) {
			Point2D point = svgPolygon.getPointList().get(i);
			javaPolygon.addPoint((int) point.getX(), (int) point.getY());
		}
	}

	@Override
	public void setDiagonal(Point2D point1, Point2D point2) {
		throw new UnsupportedOperationException();
	}

	@Override
	protected boolean containsUntransformed(Point2D p, double scale) {
		return javaPolygon.contains(p);

	}

	@Override
	public SVGPolygon getModelElement() {
		return svgPolygon;
	}

	@Override
	protected String getName() {
		return "polygon";
	}

	@Override
	public Shape getShape() {
		return javaPolygon;
	}

	@Override
	public void addAllMarkers() {
		Toolkit tk = Toolkit.getDefaultToolkit();
		Image img = tk.getImage(ClassLoader
				.getSystemResource("rotate cursor.gif"));
		Cursor cursor = tk.createCustomCursor(img, new Point(12, 12), "rotate");
		getMarkers().add(
				new PolyRotateMarker(this, getComponent(), cursor, true));
		for (int i = 0; i < getModelElement().getPointList().size(); i++) {
			getMarkers().add(new PolygonMarker(this, i));
		}

	}

	@Override
	public Rectangle2D getMinRect() {
		List<Point2D> list = getModelElement().getPointList();
		double minX = list.get(0).getX();
		double maxX = list.get(0).getX();
		double minY = list.get(0).getY();
		double maxY = list.get(0).getY();
		for (Point2D point : list) {
			minX = Math.min(minX, point.getX());
			maxX = Math.max(maxX, point.getX());
			minY = Math.min(minY, point.getY());
			maxY = Math.max(maxY, point.getY());
		}
		minRect.setFrame(minX, minY, maxX - minX, maxY - minY);
		return minRect;
	}

	@Override
	protected void createElement(Element element) {
		super.createElement(element);
		StringBuffer buffer = new StringBuffer();
		for (int i = 0; i < getModelElement().getPointList().size(); i++) {
			Point2D point = getModelElement().getPointList().get(i);
			buffer.append(point.getX() + "," + point.getY() + " ");
		}
		element.setAttribute("points", buffer.toString());
	}

	public void setCorner(int n, Point2D p) {
		getModelElement().setCorner(n, p);
		javaPolygon.xpoints[n] = (int) (p.getX());
		javaPolygon.ypoints[n] = (int) (p.getY());
	}

}
