package ru.amse.fedorov.plainsvg.presentation.factories;

import java.awt.Color;
import java.awt.geom.AffineTransform;
import java.util.HashMap;
import java.util.Map;

import org.w3c.dom.Element;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.model.elements.SVGElement;
import ru.amse.fedorov.plainsvg.presentation.elements.ElementPresentation;

public abstract class ElementFactory<E extends ElementPresentation<?>> {

	public abstract E createElement(Element element, GraphicsComponent component);

	public static Map<String, Color> map = new HashMap<String, Color>();

	static {
		map.put("black", Color.BLACK);
		map.put("blue", Color.BLUE);
		map.put("cyan", Color.CYAN);
		map.put("dark-gray", Color.DARK_GRAY);
		map.put("gray", Color.GRAY);
		map.put("green", Color.GREEN);
		map.put("light-gray", Color.LIGHT_GRAY);
		map.put("magenta", Color.MAGENTA);
		map.put("orange", Color.ORANGE);
		map.put("pink", Color.PINK);
		map.put("red", Color.RED);
		map.put("white", Color.WHITE);
		map.put("yellow", Color.YELLOW);
		map.put("lime", Color.GREEN);
	}

	public static Color getColor(String s) {
		if ((s.isEmpty()) || (s.equals("none"))) {
			return null;
		} else if (map.containsKey(s)) {
			return map.get(s);
		} else {
			s = s.substring(1, s.length());
			try {
				return new Color(Integer.valueOf(s, 16));
			} catch (NumberFormatException e) {
				return Color.BLACK;
			}
		}
	}

	private static AffineTransform getTransform(String s) {
		if (s.isEmpty()) {
			return new AffineTransform();
		} else {
			s = s.substring(7, s.length() - 1);
			double[] ar = new double[6];
			String[] arString = s.split(" |\n");
			int k = 0;
			for (int i = 0; i < arString.length; i++) {
				if (!arString[i].isEmpty()) {
					ar[k] = Double.parseDouble(arString[i]);
					k++;
				}

			}
			return new AffineTransform(ar[0], ar[1], ar[2], ar[3], ar[4], ar[5]);
		}
	}

	protected void setParameters(SVGElement svgElement, Element element,
			GraphicsComponent component) {
		String transform = element.getAttribute("transform");
		svgElement.setAffineTransform(getTransform(transform));
		svgElement.addElementChangedListener(component.getSuperListener());
	}

}
