package ru.amse.fedorov.plainsvg.tools;

import java.awt.Cursor;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.model.elements.SVGLine;
import ru.amse.fedorov.plainsvg.model.elements.SVGPolygon;
import ru.amse.fedorov.plainsvg.presentation.elements.LinePresentation;
import ru.amse.fedorov.plainsvg.presentation.elements.PolygonPresentation;

public class CreatePolygonTool extends
		CreateTool<SVGPolygon, PolygonPresentation> {

	/*
	 * Mediator for created ellipse.
	 */
	private PolygonPresentation polygonPresentation;

	/*
	 * Created svg-ellipse
	 */
	private SVGPolygon svgPolygon;

	private final List<LinePresentation> lines = new ArrayList<LinePresentation>();

	public CreatePolygonTool(GraphicsComponent component) {
		super(component);
	}

	@Override
	protected SVGPolygon getElement() {
		return svgPolygon;
	}

	@Override
	protected PolygonPresentation getPresentation() {
		return polygonPresentation;
	}

	@Override
	protected boolean isElementSingular() {
		return lines.size() < 3;
	}

	@Override
	public void mouseMoved(Point2D point, boolean isControlDown) {
		createPolyline(point, isControlDown);
	}

	private void createPolyline(Point2D point, boolean isControlDown) {
		if (!lines.isEmpty()) {
			Point2D firstPoint = lines.get(0).getModelElement().getPoint1();
			if (firstPoint.distance(point) > 5 / getComponent().getScale()) {
				getComponent().setCursor(
						Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
			} else {
				getComponent().setCursor(
						Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			}
			LinePresentation currentLine = lines.get(lines.size() - 1);
			currentLine.setDiagonal(currentLine.getModelElement().getPoint1(),
					point);
		}

	}

	@Override
	public void mouseDragged(Point2D point, boolean isControlDown) {
		createPolyline(point, isControlDown);
	}

	@Override
	public void mouseReleased(Point2D point, boolean isControlDown) {
		if (!lines.isEmpty()) {
			Point2D firstPoint = lines.get(0).getModelElement().getPoint1();
			if (firstPoint.distance(point) > 5 / getComponent().getScale()) {
				addNewLine(point);
			} else {
				List<Point2D> points = new ArrayList<Point2D>(lines.size());
				for (int i = 0; i < lines.size(); i++) {
					points.add(lines.get(i).getModelElement().getPoint1());
				}
				svgPolygon = new SVGPolygon(points);
				PolygonPresentation pres = new PolygonPresentation(svgPolygon,
						getComponent());
				getElementList().add(pres);
				svgPolygon.addElementChangedListener(getComponent()
						.getSuperListener());
				getElementList().removeAll(lines);
				pres.addMarkers();
				getSelectedList().add(pres);
				getComponent().setDefaultCurrentTool();
				if (isElementSingular()) {
					getSelectedList().remove(pres);
					getElementList().remove(pres);
				}
				lines.clear();
			}
		} else {
			getSelectedList().clear();
			addNewLine(point);
		}
	}

	private void addNewLine(Point2D point) {
		SVGLine line = new SVGLine(point);
		line.addElementChangedListener(getComponent().getSuperListener());
		LinePresentation pres = new LinePresentation(line);
		lines.add(pres);
		getElementList().add(pres);
		pres.addMarkers();
	}
}
