package ru.amse.fedorov.plainsvg.tools;

import java.awt.geom.Point2D;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.model.elements.SVGRectangle;
import ru.amse.fedorov.plainsvg.presentation.elements.RectanglePresentation;

/**
 * Tool for creating rectangles.
 * 
 * @author Alexey Fedorov
 */
public class CreateRectangleTool extends CreateDiagonaledTool<SVGRectangle, RectanglePresentation> {

	/*
	 * Mediator for created rectangle.
	 */
	private RectanglePresentation rectanglePresentation;

	/*
	 * Created svg-rectangle.
	 */
	private SVGRectangle svgRectangle;

	/**
	 * Creates instance of tool "Create rectangle".
	 * 
	 * @param component
	 *            graphics component to draw the rectangle
	 */
	public CreateRectangleTool(GraphicsComponent component) {
		super(component);
	}

	@Override
	protected void createElement(Point2D point) {
		svgRectangle = new SVGRectangle(getPointWherePressed());
		rectanglePresentation = new RectanglePresentation(svgRectangle,
				getComponent());
	}

	@Override
	protected RectanglePresentation getPresentation() {
		return rectanglePresentation;
	}

	@Override
	protected SVGRectangle getElement() {
		return svgRectangle;
	}

	@Override
	protected boolean isElementSingular() {
		return (Math.abs(svgRectangle.getHeight()) < EPS)
				&& (Math.abs(svgRectangle.getWidth()) < EPS);
	}

}
