package ru.amse.fedorov.plainsvg.tools;

import java.awt.Cursor;
import java.awt.geom.Point2D;

import ru.amse.fedorov.plainsvg.gui.GraphicsComponent;
import ru.amse.fedorov.plainsvg.model.elements.SVGElement;
import ru.amse.fedorov.plainsvg.presentation.elements.StrokedPresentation;

/**
 * Base class for all creating tools.
 * 
 * @author Alexey Fedorov
 */
public abstract class CreateTool<E extends SVGElement, P extends StrokedPresentation<?>>
		extends ToolAdapter {

	/**
	 * Maximal absolute accuracy of measuring
	 */
	public static final double EPS = 0.000000001;

	/*
	 * The point when mouse was pressed.
	 */
	private final Point2D pointWherePressed = new Point2D.Double();

	/**
	 * Creates creating tool base on master graphics component.
	 * 
	 * @param component
	 *            master graphics component
	 * 
	 * @see ru.amse.fedorov.plainsvg.gui.GraphicsComponent
	 */
	/* package */CreateTool(GraphicsComponent component) {
		super(component);
	}

	/**
	 * @return presenation for the given svg-element
	 */
	protected abstract P getPresentation();

	/**
	 * @return given svg-element
	 */
	protected abstract E getElement();

	/**
	 * @return TRUE if creating element is singular
	 */
	protected abstract boolean isElementSingular();

	/**
	 * @return the pointWherePressed
	 */
	protected Point2D getPointWherePressed() {
		return pointWherePressed;
	}

	@Override
	public void mouseClicked(Point2D point, boolean isControlDown) {
		// Nothing !
	}

	@Override
	public void mouseMoved(Point2D point, boolean isControlDown) {
		getComponent().setCursor(
				Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
	}
}
